@@ -1,31 +1,24 @@
-# -*- coding: utf-8 -*-
-"""
-    celery.bootsteps
-    ~~~~~~~~~~~~~~~~
-
-    A directed acyclic graph of reusable components.
-
-"""
-from __future__ import absolute_import, unicode_literals
+"""A directed acyclic graph of reusable components."""
 
 from collections import deque
 from threading import Event
 
 from kombu.common import ignore_errors
-from kombu.utils import symbol_by_name
+from kombu.utils.encoding import bytes_to_str
+from kombu.utils.imports import symbol_by_name
 
-from .datastructures import DependencyGraph, GraphFormatter
-from .five import values, with_metaclass
+from .utils.graph import DependencyGraph, GraphFormatter
 from .utils.imports import instantiate, qualname
 from .utils.log import get_logger
 
 try:
     from greenlet import GreenletExit
-    IGNORE_ERRORS = (GreenletExit, )
 except ImportError:  # pragma: no cover
     IGNORE_ERRORS = ()
+else:
+    IGNORE_ERRORS = (GreenletExit,)
 
-__all__ = ['Blueprint', 'Step', 'StartStopStep', 'ConsumerStep']
+__all__ = ('Blueprint', 'Step', 'StartStopStep', 'ConsumerStep')
 
 #: States
 RUN = 0x1
@@ -33,11 +26,10 @@ CLOSE = 0x2
 TERMINATE = 0x3
 
 logger = get_logger(__name__)
-debug = logger.debug
 
 
 def _pre(ns, fmt):
-    return '| {0}: {1}'.format(ns.alias, fmt)
+    return f'| {ns.alias}: {fmt}'
 
 
 def _label(s):
@@ -56,9 +48,10 @@ class StepFormatter(GraphFormatter):
     }
 
     def label(self, step):
-        return step and '{0}{1}'.format(
+        return step and '{}{}'.format(
             self._get_prefix(step),
-            (step.label or _label(step)).encode('utf-8', 'ignore'),
+            bytes_to_str(
+                (step.label or _label(step)).encode('utf-8', 'ignore')),
         )
 
     def _get_prefix(self, step):
@@ -78,17 +71,18 @@ class StepFormatter(GraphFormatter):
         return self.draw_edge(a, b, self.edge_scheme, attrs)
 
 
-class Blueprint(object):
+class Blueprint:
     """Blueprint containing bootsteps that can be applied to objects.
 
-    :keyword steps: List of steps.
-    :keyword name: Set explicit name for this blueprint.
-    :keyword app: Set the Celery app for this blueprint.
-    :keyword on_start: Optional callback applied after blueprint start.
-    :keyword on_close: Optional callback applied before blueprint close.
-    :keyword on_stopped: Optional callback applied after blueprint stopped.
-
+    Arguments:
+        steps Sequence[Union[str, Step]]: List of steps.
+        name (str): Set explicit name for this blueprint.
+        on_start (Callable): Optional callback applied after blueprint start.
+        on_close (Callable): Optional callback applied before blueprint close.
+        on_stopped (Callable): Optional callback applied after
+            blueprint stopped.
     """
+
     GraphFormatter = StepFormatter
 
     name = None
@@ -102,9 +96,8 @@ class Blueprint(object):
         TERMINATE: 'terminating',
     }
 
-    def __init__(self, steps=None, name=None, app=None,
+    def __init__(self, steps=None, name=None,
                  on_start=None, on_close=None, on_stopped=None):
-        self.app = app
         self.name = name or self.name or qualname(type(self))
         self.types = set(steps or []) | set(self.default_steps)
         self.on_start = on_start
@@ -121,7 +114,7 @@ class Blueprint(object):
             self._debug('Starting %s', step.alias)
             self.started = i + 1
             step.start(parent)
-            debug('^-- substep ok')
+            logger.debug('^-- substep ok')
 
     def human_state(self):
         return self.state_to_name[self.state or 0]
@@ -153,13 +146,11 @@ class Blueprint(object):
                                 description.capitalize(), step.alias)
                     try:
                         fun(parent, *args)
-                    except Exception as exc:
+                    except Exception as exc:  # pylint: disable=broad-except
                         if propagate:
                             raise
-                        logger.error(
-                            'Error on %s %s: %r',
-                            description, step.alias, exc, exc_info=1,
-                        )
+                        logger.exception(
+                            'Error on %s %s: %r', description, step.alias, exc)
 
     def stop(self, parent, close=True, terminate=False):
         what = 'terminating' if terminate else 'stopping'
@@ -204,7 +195,6 @@ class Blueprint(object):
 
         For :class:`StartStopStep` the services created
         will also be added to the objects ``steps`` attribute.
-
         """
         self._debug('Preparing bootsteps.')
         order = self.order = []
@@ -229,10 +219,12 @@ class Blueprint(object):
         return self.steps[name]
 
     def _find_last(self):
-        return next((C for C in values(self.steps) if C.last), None)
+        return next((C for C in self.steps.values() if C.last), None)
 
     def _firstpass(self, steps):
-        stream = deque(step.requires for step in values(steps))
+        for step in steps.values():
+            step.requires = [symbol_by_name(dep) for dep in step.requires]
+        stream = deque(step.requires for step in steps.values())
         while stream:
             for node in stream.popleft():
                 node = symbol_by_name(node)
@@ -243,7 +235,7 @@ class Blueprint(object):
     def _finalize_steps(self, steps):
         last = self._find_last()
         self._firstpass(steps)
-        it = ((C, C.requires) for C in values(steps))
+        it = ((C, C.requires) for C in steps.values())
         G = self.graph = DependencyGraph(
             it, formatter=self.GraphFormatter(root=last),
         )
@@ -257,17 +249,14 @@ class Blueprint(object):
             raise KeyError('unknown bootstep: %s' % exc)
 
     def claim_steps(self):
-        return dict(self.load_step(step) for step in self._all_steps())
-
-    def _all_steps(self):
-        return self.types | self.app.steps[self.name.lower()]
+        return dict(self.load_step(step) for step in self.types)
 
     def load_step(self, step):
         step = symbol_by_name(step)
         return step.name, step
 
     def _debug(self, msg, *args):
-        return debug(_pre(self, msg), *args)
+        return logger.debug(_pre(self, msg), *args)
 
     @property
     def alias(self):
@@ -275,37 +264,37 @@ class Blueprint(object):
 
 
 class StepType(type):
-    """Metaclass for steps."""
+    """Meta-class for steps."""
+
+    name = None
+    requires = None
 
     def __new__(cls, name, bases, attrs):
         module = attrs.get('__module__')
-        qname = '{0}.{1}'.format(module, name) if module else name
+        qname = f'{module}.{name}' if module else name
         attrs.update(
             __qualname__=qname,
             name=attrs.get('name') or qname,
-            requires=attrs.get('requires', ()),
         )
-        return super(StepType, cls).__new__(cls, name, bases, attrs)
+        return super().__new__(cls, name, bases, attrs)
 
-    def __str__(self):
-        return self.name
+    def __str__(cls):
+        return cls.name
 
-    def __repr__(self):
-        return 'step:{0.name}{{{0.requires!r}}}'.format(self)
+    def __repr__(cls):
+        return 'step:{0.name}{{{0.requires!r}}}'.format(cls)
 
 
-@with_metaclass(StepType)
-class Step(object):
+class Step(metaclass=StepType):
     """A Bootstep.
 
     The :meth:`__init__` method is called when the step
     is bound to a parent object, and can as such be used
     to initialize attributes in the parent object at
     parent instantiation-time.
-
     """
 
-    #: Optional step name, will use qualname if not specified.
+    #: Optional step name, will use ``qualname`` if not specified.
     name = None
 
     #: Optional short name used for graph outputs and in logs.
@@ -331,8 +320,11 @@ class Step(object):
         pass
 
     def include_if(self, parent):
-        """An optional predicate that decides whether this
-        step should be created."""
+        """Return true if bootstep should be included.
+
+        You can define this as an optional predicate that decides whether
+        this step should be created.
+        """
         return self.enabled
 
     def instantiate(self, name, *args, **kwargs):
@@ -348,10 +340,9 @@ class Step(object):
 
     def create(self, parent):
         """Create the step."""
-        pass
 
     def __repr__(self):
-        return '<step: {0.alias}>'.format(self)
+        return f'<step: {self.alias}>'
 
     @property
     def alias(self):
@@ -362,6 +353,7 @@ class Step(object):
 
 
 class StartStopStep(Step):
+    """Bootstep that must be started and stopped in order."""
 
     #: Optional obj created by the :meth:`create` method.
     #: This is used by :class:`StartStopStep` to keep the
@@ -392,7 +384,9 @@ class StartStopStep(Step):
 
 
 class ConsumerStep(StartStopStep):
-    requires = ('celery.worker.consumer:Connection', )
+    """Bootstep that starts a message consumer."""
+
+    requires = ('celery.worker.consumer:Connection',)
     consumers = None
 
     def get_consumers(self, channel):
