@@ -2,7 +2,7 @@
 
 Usage::
 
-   (window1)$ python myapp.py worker -l info
+   (window1)$ python myapp.py worker -l INFO
 
    (window2)$ python
    >>> from myapp import add
@@ -13,13 +13,13 @@ Usage::
 You can also specify the app to use with the `celery` command,
 using the `-A` / `--app` option::
 
-    $ celery -A myapp worker -l info
+    $ celery -A myapp worker -l INFO
 
 With the `-A myproj` argument the program will search for an app
 instance in the module ``myproj``.  You can also specify an explicit
 name using the fully qualified form::
 
-    $ celery -A myapp:app worker -l info
+    $ celery -A myapp:app worker -l INFO
 
 """
 
