@@ -1,3 +1,4 @@
+import itertools
 import json
 import random
 import ssl
@@ -7,12 +8,15 @@ from pickle import dumps, loads
 from unittest.mock import ANY, Mock, call, patch
 
 import pytest
-from case import ContextMock, mock
 
 from celery import signature, states, uuid
 from celery.canvas import Signature
-from celery.exceptions import ChordError, ImproperlyConfigured
+from celery.contrib.testing.mocks import ContextMock
+from celery.exceptions import (BackendStoreError, ChordError,
+                               ImproperlyConfigured)
+from celery.result import AsyncResult, GroupResult
 from celery.utils.collections import AttributeDict
+from t.unit import conftest
 
 
 def raise_on_second_call(mock, exc, *retval):
@@ -58,7 +62,7 @@ class Pipeline:
         return [step(*a, **kw) for step, a, kw in self.steps]
 
 
-class PubSub(mock.MockCallbacks):
+class PubSub(conftest.MockCallbacks):
     def __init__(self, ignore_subscribe_messages=False):
         self._subscribed_to = set()
 
@@ -75,7 +79,7 @@ class PubSub(mock.MockCallbacks):
         pass
 
 
-class Redis(mock.MockCallbacks):
+class Redis(conftest.MockCallbacks):
     Connection = Connection
     Pipeline = Pipeline
     pubsub = PubSub
@@ -140,7 +144,7 @@ class Redis(mock.MockCallbacks):
 
     def zrange(self, key, start, stop):
         # `stop` is inclusive in Redis so we use `stop + 1` unless that would
-        # cause us to move from negative (right-most) indicies to positive
+        # cause us to move from negative (right-most) indices to positive
         stop = stop + 1 if stop != -1 else None
         return [e[1] for e in self._get_sorted_set(key)[start:stop]]
 
@@ -155,7 +159,7 @@ class Redis(mock.MockCallbacks):
         return len(self.zrangebyscore(key, min_, max_))
 
 
-class Sentinel(mock.MockCallbacks):
+class Sentinel(conftest.MockCallbacks):
     def __init__(self, sentinels, min_other_sentinels=0, sentinel_kwargs=None,
                  **connection_kwargs):
         self.sentinel_kwargs = sentinel_kwargs
@@ -273,8 +277,17 @@ class test_RedisResultConsumer:
         parent_on_state_change.assert_called_with(meta, None)
         assert consumer._pubsub._subscribed_to == {b'celery-task-meta-initial'}
 
+    def test_drain_events_connection_error_no_patch(self):
+        meta = {'task_id': 'initial', 'status': states.SUCCESS}
+        consumer = self.get_consumer()
+        consumer.start('initial')
+        consumer.backend._set_with_state(b'celery-task-meta-initial', json.dumps(meta), states.SUCCESS)
+        consumer._pubsub.get_message.side_effect = ConnectionError()
+        consumer.drain_events()
+        consumer._pubsub.subscribe.assert_not_called()
 
-class test_RedisBackend:
+
+class basetest_RedisBackend:
     def get_backend(self):
         from celery.backends.redis import RedisBackend
 
@@ -287,11 +300,43 @@ class test_RedisBackend:
         from celery.backends.redis import E_LOST
         return E_LOST
 
+    def create_task(self, i, group_id="group_id"):
+        tid = uuid()
+        task = Mock(name=f'task-{tid}')
+        task.name = 'foobarbaz'
+        self.app.tasks['foobarbaz'] = task
+        task.request.chord = signature(task)
+        task.request.id = tid
+        self.b.set_chord_size(group_id, 10)
+        task.request.group = group_id
+        task.request.group_index = i
+        return task
+
+    @contextmanager
+    def chord_context(self, size=1):
+        with patch('celery.backends.redis.maybe_signature') as ms:
+            request = Mock(name='request')
+            request.id = 'id1'
+            group_id = 'gid1'
+            request.group = group_id
+            request.group_index = None
+            tasks = [
+                self.create_task(i, group_id=request.group)
+                for i in range(size)
+            ]
+            callback = ms.return_value = Signature('add')
+            callback.id = 'id1'
+            self.b.set_chord_size(group_id, size)
+            callback.delay = Mock(name='callback.delay')
+            yield tasks, request, callback
+
     def setup(self):
         self.Backend = self.get_backend()
         self.E_LOST = self.get_E_LOST()
         self.b = self.Backend(app=self.app)
 
+
+class test_RedisBackend(basetest_RedisBackend):
     @pytest.mark.usefixtures('depends_on_current_app')
     def test_reduce(self):
         pytest.importorskip('redis')
@@ -305,6 +350,20 @@ class test_RedisBackend:
         with pytest.raises(ImproperlyConfigured):
             self.Backend(app=self.app)
 
+    def test_username_password_from_redis_conf(self):
+        self.app.conf.redis_password = 'password'
+        x = self.Backend(app=self.app)
+
+        assert x.connparams
+        assert 'username' not in x.connparams
+        assert x.connparams['password'] == 'password'
+        self.app.conf.redis_username = 'username'
+        x = self.Backend(app=self.app)
+
+        assert x.connparams
+        assert x.connparams['username'] == 'username'
+        assert x.connparams['password'] == 'password'
+
     def test_url(self):
         self.app.conf.redis_socket_timeout = 30.0
         self.app.conf.redis_socket_connect_timeout = 100.0
@@ -318,6 +377,19 @@ class test_RedisBackend:
         assert x.connparams['password'] == 'bosco'
         assert x.connparams['socket_timeout'] == 30.0
         assert x.connparams['socket_connect_timeout'] == 100.0
+        assert 'username' not in x.connparams
+
+        x = self.Backend(
+            'redis://username:bosco@vandelay.com:123//1', app=self.app,
+        )
+        assert x.connparams
+        assert x.connparams['host'] == 'vandelay.com'
+        assert x.connparams['db'] == 1
+        assert x.connparams['port'] == 123
+        assert x.connparams['username'] == 'username'
+        assert x.connparams['password'] == 'bosco'
+        assert x.connparams['socket_timeout'] == 30.0
+        assert x.connparams['socket_connect_timeout'] == 100.0
 
     def test_timeouts_in_url_coerced(self):
         pytest.importorskip('redis')
@@ -383,6 +455,54 @@ class test_RedisBackend:
         from redis.connection import SSLConnection
         assert x.connparams['connection_class'] is SSLConnection
 
+    def test_backend_health_check_interval_ssl(self):
+        pytest.importorskip('redis')
+
+        self.app.conf.redis_backend_use_ssl = {
+            'ssl_cert_reqs': ssl.CERT_REQUIRED,
+            'ssl_ca_certs': '/path/to/ca.crt',
+            'ssl_certfile': '/path/to/client.crt',
+            'ssl_keyfile': '/path/to/client.key',
+        }
+        self.app.conf.redis_backend_health_check_interval = 10
+        x = self.Backend(
+            'rediss://:bosco@vandelay.com:123//1', app=self.app,
+        )
+        assert x.connparams
+        assert x.connparams['host'] == 'vandelay.com'
+        assert x.connparams['db'] == 1
+        assert x.connparams['port'] == 123
+        assert x.connparams['password'] == 'bosco'
+        assert x.connparams['health_check_interval'] == 10
+
+        from redis.connection import SSLConnection
+        assert x.connparams['connection_class'] is SSLConnection
+
+    def test_backend_health_check_interval(self):
+        pytest.importorskip('redis')
+
+        self.app.conf.redis_backend_health_check_interval = 10
+        x = self.Backend(
+            'redis://vandelay.com:123//1', app=self.app,
+        )
+        assert x.connparams
+        assert x.connparams['host'] == 'vandelay.com'
+        assert x.connparams['db'] == 1
+        assert x.connparams['port'] == 123
+        assert x.connparams['health_check_interval'] == 10
+
+    def test_backend_health_check_interval_not_set(self):
+        pytest.importorskip('redis')
+
+        x = self.Backend(
+            'redis://vandelay.com:123//1', app=self.app,
+        )
+        assert x.connparams
+        assert x.connparams['host'] == 'vandelay.com'
+        assert x.connparams['db'] == 1
+        assert x.connparams['port'] == 123
+        assert "health_check_interval" not in x.connparams
+
     @pytest.mark.parametrize('cert_str', [
         "required",
         "CERT_REQUIRED",
@@ -558,11 +678,11 @@ class test_RedisBackend:
 
     def test_apply_chord(self, unlock='celery.chord_unlock'):
         self.app.tasks[unlock] = Mock()
-        header_result = self.app.GroupResult(
+        header_result_args = (
             uuid(),
             [self.app.AsyncResult(x) for x in range(3)],
         )
-        self.b.apply_chord(header_result, None)
+        self.b.apply_chord(header_result_args, None)
         assert self.app.tasks[unlock].apply_async.call_count == 0
 
     def test_unpack_chord_result(self):
@@ -607,6 +727,12 @@ class test_RedisBackend:
         b.add_to_chord(gid, 'sig')
         b.client.incr.assert_called_with(b.get_key_for_group(gid, '.t'), 1)
 
+    def test_set_chord_size(self):
+        b = self.Backend('redis://', app=self.app)
+        gid = uuid()
+        b.set_chord_size(gid, 10)
+        b.client.set.assert_called_with(b.get_key_for_group(gid, '.s'), 10)
+
     def test_expires_is_None(self):
         b = self.Backend(expires=None, app=self.app)
         assert b.expires == self.app.conf.result_expires.total_seconds()
@@ -623,20 +749,40 @@ class test_RedisBackend:
         self.b.expires = None
         self.b._set_with_state('foo', 'bar', states.SUCCESS)
 
-    def create_task(self, i):
-        tid = uuid()
-        task = Mock(name=f'task-{tid}')
-        task.name = 'foobarbaz'
-        self.app.tasks['foobarbaz'] = task
-        task.request.chord = signature(task)
-        task.request.id = tid
-        task.request.chord['chord_size'] = 10
-        task.request.group = 'group_id'
-        task.request.group_index = i
-        return task
+    def test_process_cleanup(self):
+        self.b.process_cleanup()
 
-    @patch('celery.result.GroupResult.restore')
-    def test_on_chord_part_return(self, restore):
+    def test_get_set_forget(self):
+        tid = uuid()
+        self.b.store_result(tid, 42, states.SUCCESS)
+        assert self.b.get_state(tid) == states.SUCCESS
+        assert self.b.get_result(tid) == 42
+        self.b.forget(tid)
+        assert self.b.get_state(tid) == states.PENDING
+
+    def test_set_expires(self):
+        self.b = self.Backend(expires=512, app=self.app)
+        tid = uuid()
+        key = self.b.get_key_for_task(tid)
+        self.b.store_result(tid, 42, states.SUCCESS)
+        self.b.client.expire.assert_called_with(
+            key, 512,
+        )
+
+    def test_set_raises_error_on_large_value(self):
+        with pytest.raises(BackendStoreError):
+            self.b.set('key', 'x' * (self.b._MAX_STR_VALUE_SIZE + 1))
+
+
+class test_RedisBackend_chords_simple(basetest_RedisBackend):
+    @pytest.fixture(scope="class", autouse=True)
+    def simple_header_result(self):
+        with patch(
+            "celery.result.GroupResult.restore", return_value=None,
+        ) as p:
+            yield p
+
+    def test_on_chord_part_return(self):
         tasks = [self.create_task(i) for i in range(10)]
         random.shuffle(tasks)
 
@@ -647,13 +793,13 @@ class test_RedisBackend:
         assert self.b.client.zrangebyscore.call_count
         jkey = self.b.get_key_for_group('group_id', '.j')
         tkey = self.b.get_key_for_group('group_id', '.t')
-        self.b.client.delete.assert_has_calls([call(jkey), call(tkey)])
+        skey = self.b.get_key_for_group('group_id', '.s')
+        self.b.client.delete.assert_has_calls([call(jkey), call(tkey), call(skey)])
         self.b.client.expire.assert_has_calls([
-            call(jkey, 86400), call(tkey, 86400),
+            call(jkey, 86400), call(tkey, 86400), call(skey, 86400),
         ])
 
-    @patch('celery.result.GroupResult.restore')
-    def test_on_chord_part_return__unordered(self, restore):
+    def test_on_chord_part_return__unordered(self):
         self.app.conf.result_backend_transport_options = dict(
             result_chord_ordered=False,
         )
@@ -673,8 +819,7 @@ class test_RedisBackend:
             call(jkey, 86400), call(tkey, 86400),
         ])
 
-    @patch('celery.result.GroupResult.restore')
-    def test_on_chord_part_return__ordered(self, restore):
+    def test_on_chord_part_return__ordered(self):
         self.app.conf.result_backend_transport_options = dict(
             result_chord_ordered=True,
         )
@@ -694,11 +839,28 @@ class test_RedisBackend:
             call(jkey, 86400), call(tkey, 86400),
         ])
 
-    @patch('celery.result.GroupResult.restore')
-    def test_on_chord_part_return_no_expiry(self, restore):
+    def test_on_chord_part_return_no_expiry(self):
         old_expires = self.b.expires
         self.b.expires = None
         tasks = [self.create_task(i) for i in range(10)]
+        self.b.set_chord_size('group_id', 10)
+
+        for i in range(10):
+            self.b.on_chord_part_return(tasks[i].request, states.SUCCESS, i)
+            assert self.b.client.zadd.call_count
+            self.b.client.zadd.reset_mock()
+        assert self.b.client.zrangebyscore.call_count
+        jkey = self.b.get_key_for_group('group_id', '.j')
+        tkey = self.b.get_key_for_group('group_id', '.t')
+        self.b.client.delete.assert_has_calls([call(jkey), call(tkey)])
+        self.b.client.expire.assert_not_called()
+
+        self.b.expires = old_expires
+
+    def test_on_chord_part_return_expire_set_to_zero(self):
+        old_expires = self.b.expires
+        self.b.expires = 0
+        tasks = [self.create_task(i) for i in range(10)]
 
         for i in range(10):
             self.b.on_chord_part_return(tasks[i].request, states.SUCCESS, i)
@@ -712,8 +874,7 @@ class test_RedisBackend:
 
         self.b.expires = old_expires
 
-    @patch('celery.result.GroupResult.restore')
-    def test_on_chord_part_return_no_expiry__unordered(self, restore):
+    def test_on_chord_part_return_no_expiry__unordered(self):
         self.app.conf.result_backend_transport_options = dict(
             result_chord_ordered=False,
         )
@@ -734,8 +895,7 @@ class test_RedisBackend:
 
         self.b.expires = old_expires
 
-    @patch('celery.result.GroupResult.restore')
-    def test_on_chord_part_return_no_expiry__ordered(self, restore):
+    def test_on_chord_part_return_no_expiry__ordered(self):
         self.app.conf.result_backend_transport_options = dict(
             result_chord_ordered=True,
         )
@@ -824,8 +984,8 @@ class test_RedisBackend:
         with self.chord_context(1) as (_, request, callback):
             self.b.client.pipeline = ContextMock()
             raise_on_second_call(self.b.client.pipeline, ChordError())
-            self.b.client.pipeline.return_value.zadd().zcount().get().expire(
-            ).expire().execute.return_value = (1, 1, 0, 4, 5)
+            self.b.client.pipeline.return_value.zadd().zcount().get().get().expire(
+            ).expire().expire().execute.return_value = (1, 1, 0, b'1', 4, 5, 6)
             task = self.app._tasks['add'] = Mock(name='add_task')
             self.b.on_chord_part_return(request, states.SUCCESS, 10)
             task.backend.fail_from_current_stack.assert_called_with(
@@ -840,8 +1000,8 @@ class test_RedisBackend:
         with self.chord_context(1) as (_, request, callback):
             self.b.client.pipeline = ContextMock()
             raise_on_second_call(self.b.client.pipeline, ChordError())
-            self.b.client.pipeline.return_value.rpush().llen().get().expire(
-            ).expire().execute.return_value = (1, 1, 0, 4, 5)
+            self.b.client.pipeline.return_value.rpush().llen().get().get().expire(
+            ).expire().expire().execute.return_value = (1, 1, 0, b'1', 4, 5, 6)
             task = self.app._tasks['add'] = Mock(name='add_task')
             self.b.on_chord_part_return(request, states.SUCCESS, 10)
             task.backend.fail_from_current_stack.assert_called_with(
@@ -856,8 +1016,8 @@ class test_RedisBackend:
         with self.chord_context(1) as (_, request, callback):
             self.b.client.pipeline = ContextMock()
             raise_on_second_call(self.b.client.pipeline, ChordError())
-            self.b.client.pipeline.return_value.zadd().zcount().get().expire(
-            ).expire().execute.return_value = (1, 1, 0, 4, 5)
+            self.b.client.pipeline.return_value.zadd().zcount().get().get().expire(
+            ).expire().expire().execute.return_value = (1, 1, 0, b'1', 4, 5, 6)
             task = self.app._tasks['add'] = Mock(name='add_task')
             self.b.on_chord_part_return(request, states.SUCCESS, 10)
             task.backend.fail_from_current_stack.assert_called_with(
@@ -868,8 +1028,8 @@ class test_RedisBackend:
         with self.chord_context(1) as (_, request, callback):
             self.b.client.pipeline = ContextMock()
             raise_on_second_call(self.b.client.pipeline, RuntimeError())
-            self.b.client.pipeline.return_value.zadd().zcount().get().expire(
-            ).expire().execute.return_value = (1, 1, 0, 4, 5)
+            self.b.client.pipeline.return_value.zadd().zcount().get().get().expire(
+            ).expire().expire().execute.return_value = (1, 1, 0, b'1', 4, 5, 6)
             task = self.app._tasks['add'] = Mock(name='add_task')
             self.b.on_chord_part_return(request, states.SUCCESS, 10)
             task.backend.fail_from_current_stack.assert_called_with(
@@ -884,8 +1044,8 @@ class test_RedisBackend:
         with self.chord_context(1) as (_, request, callback):
             self.b.client.pipeline = ContextMock()
             raise_on_second_call(self.b.client.pipeline, RuntimeError())
-            self.b.client.pipeline.return_value.rpush().llen().get().expire(
-            ).expire().execute.return_value = (1, 1, 0, 4, 5)
+            self.b.client.pipeline.return_value.rpush().llen().get().get().expire(
+            ).expire().expire().execute.return_value = (1, 1, 0, b'1', 4, 5, 6)
             task = self.app._tasks['add'] = Mock(name='add_task')
             self.b.on_chord_part_return(request, states.SUCCESS, 10)
             task.backend.fail_from_current_stack.assert_called_with(
@@ -900,47 +1060,95 @@ class test_RedisBackend:
         with self.chord_context(1) as (_, request, callback):
             self.b.client.pipeline = ContextMock()
             raise_on_second_call(self.b.client.pipeline, RuntimeError())
-            self.b.client.pipeline.return_value.zadd().zcount().get().expire(
-            ).expire().execute.return_value = (1, 1, 0, 4, 5)
+            self.b.client.pipeline.return_value.zadd().zcount().get().get().expire(
+            ).expire().expire().execute.return_value = (1, 1, 0, b'1', 4, 5, 6)
             task = self.app._tasks['add'] = Mock(name='add_task')
             self.b.on_chord_part_return(request, states.SUCCESS, 10)
             task.backend.fail_from_current_stack.assert_called_with(
                 callback.id, exc=ANY,
             )
 
-    @contextmanager
-    def chord_context(self, size=1):
-        with patch('celery.backends.redis.maybe_signature') as ms:
-            tasks = [self.create_task(i) for i in range(size)]
-            request = Mock(name='request')
-            request.id = 'id1'
-            request.group = 'gid1'
-            request.group_index = None
-            callback = ms.return_value = Signature('add')
-            callback.id = 'id1'
-            callback['chord_size'] = size
-            callback.delay = Mock(name='callback.delay')
-            yield tasks, request, callback
 
-    def test_process_cleanup(self):
-        self.b.process_cleanup()
+class test_RedisBackend_chords_complex(basetest_RedisBackend):
+    @pytest.fixture(scope="function", autouse=True)
+    def complex_header_result(self):
+        with patch("celery.result.GroupResult.restore") as p:
+            yield p
 
-    def test_get_set_forget(self):
-        tid = uuid()
-        self.b.store_result(tid, 42, states.SUCCESS)
-        assert self.b.get_state(tid) == states.SUCCESS
-        assert self.b.get_result(tid) == 42
-        self.b.forget(tid)
-        assert self.b.get_state(tid) == states.PENDING
+    @pytest.mark.parametrize(['results', 'assert_save_called'], [
+        # No results in the header at all - won't call `save()`
+        (tuple(), False),
+        # Simple results in the header - won't call `save()`
+        ((AsyncResult("foo"), ), False),
+        # Many simple results in the header - won't call `save()`
+        ((AsyncResult("foo"), ) * 42, False),
+        # A single complex result in the header - will call `save()`
+        ((GroupResult("foo", []),), True),
+        # Many complex results in the header - will call `save()`
+        ((GroupResult("foo"), ) * 42, True),
+        # Mixed simple and complex results in the header - will call `save()`
+        (itertools.islice(
+            itertools.cycle((
+                AsyncResult("foo"), GroupResult("foo"),
+            )), 42,
+        ), True),
+    ])
+    def test_apply_chord_complex_header(self, results, assert_save_called):
+        mock_group_result = Mock()
+        mock_group_result.return_value.results = results
+        self.app.GroupResult = mock_group_result
+        header_result_args = ("gid11", results)
+        self.b.apply_chord(header_result_args, None)
+        if assert_save_called:
+            mock_group_result.return_value.save.assert_called_once_with(backend=self.b)
+        else:
+            mock_group_result.return_value.save.assert_not_called()
 
-    def test_set_expires(self):
-        self.b = self.Backend(expires=512, app=self.app)
-        tid = uuid()
-        key = self.b.get_key_for_task(tid)
-        self.b.store_result(tid, 42, states.SUCCESS)
-        self.b.client.expire.assert_called_with(
-            key, 512,
-        )
+    def test_on_chord_part_return_timeout(self, complex_header_result):
+        tasks = [self.create_task(i) for i in range(10)]
+        random.shuffle(tasks)
+        try:
+            self.app.conf.result_chord_join_timeout += 1.0
+            for task, result_val in zip(tasks, itertools.cycle((42, ))):
+                self.b.on_chord_part_return(
+                    task.request, states.SUCCESS, result_val,
+                )
+        finally:
+            self.app.conf.result_chord_join_timeout -= 1.0
+
+        join_func = complex_header_result.return_value.join_native
+        join_func.assert_called_once_with(timeout=4.0, propagate=True)
+
+    @pytest.mark.parametrize("supports_native_join", (True, False))
+    def test_on_chord_part_return(
+        self, complex_header_result, supports_native_join,
+    ):
+        mock_result_obj = complex_header_result.return_value
+        mock_result_obj.supports_native_join = supports_native_join
+
+        tasks = [self.create_task(i) for i in range(10)]
+        random.shuffle(tasks)
+
+        with self.chord_context(10) as (tasks, request, callback):
+            for task, result_val in zip(tasks, itertools.cycle((42, ))):
+                self.b.on_chord_part_return(
+                    task.request, states.SUCCESS, result_val,
+                )
+                # Confirm that `zadd` was called even though we won't end up
+                # using the data pushed into the sorted set
+                assert self.b.client.zadd.call_count == 1
+                self.b.client.zadd.reset_mock()
+        # Confirm that neither `zrange` not `lrange` were called
+        self.b.client.zrange.assert_not_called()
+        self.b.client.lrange.assert_not_called()
+        # Confirm that the `GroupResult.restore` mock was called
+        complex_header_result.assert_called_once_with(request.group)
+        # Confirm the the callback was called with the `join()`ed group result
+        if supports_native_join:
+            expected_join = mock_result_obj.join_native
+        else:
+            expected_join = mock_result_obj.join
+        callback.delay.assert_called_once_with(expected_join())
 
 
 class test_SentinelBackend:
@@ -1005,6 +1213,16 @@ class test_SentinelBackend:
         found_dbs = [cp['db'] for cp in x.connparams['hosts']]
         assert found_dbs == expected_dbs
 
+        # By default passwords should be sanitized
+        display_url = x.as_uri()
+        assert "test" not in display_url
+        # We can choose not to sanitize with the `include_password` argument
+        unsanitized_display_url = x.as_uri(include_password=True)
+        assert unsanitized_display_url == x.url
+        # or to explicitly sanitize
+        forcibly_sanitized_display_url = x.as_uri(include_password=False)
+        assert forcibly_sanitized_display_url == display_url
+
     def test_get_sentinel_instance(self):
         x = self.Backend(
             'sentinel://:test@github.com:123/1;'
@@ -1025,3 +1243,34 @@ class test_SentinelBackend:
         )
         pool = x._get_pool(**x.connparams)
         assert pool
+
+    def test_backend_ssl(self):
+        pytest.importorskip('redis')
+
+        from celery.backends.redis import SentinelBackend
+        self.app.conf.redis_backend_use_ssl = {
+            'ssl_cert_reqs': "CERT_REQUIRED",
+            'ssl_ca_certs': '/path/to/ca.crt',
+            'ssl_certfile': '/path/to/client.crt',
+            'ssl_keyfile': '/path/to/client.key',
+        }
+        self.app.conf.redis_socket_timeout = 30.0
+        self.app.conf.redis_socket_connect_timeout = 100.0
+        x = SentinelBackend(
+            'sentinel://:bosco@vandelay.com:123//1', app=self.app,
+        )
+        assert x.connparams
+        assert len(x.connparams['hosts']) == 1
+        assert x.connparams['hosts'][0]['host'] == 'vandelay.com'
+        assert x.connparams['hosts'][0]['db'] == 1
+        assert x.connparams['hosts'][0]['port'] == 123
+        assert x.connparams['hosts'][0]['password'] == 'bosco'
+        assert x.connparams['socket_timeout'] == 30.0
+        assert x.connparams['socket_connect_timeout'] == 100.0
+        assert x.connparams['ssl_cert_reqs'] == ssl.CERT_REQUIRED
+        assert x.connparams['ssl_ca_certs'] == '/path/to/ca.crt'
+        assert x.connparams['ssl_certfile'] == '/path/to/client.crt'
+        assert x.connparams['ssl_keyfile'] == '/path/to/client.key'
+
+        from celery.backends.redis import SentinelManagedSSLConnection
+        assert x.connparams['connection_class'] is SentinelManagedSSLConnection
