@@ -3,10 +3,10 @@ import os
 from unittest.mock import Mock, patch
 
 import pytest
-from case import mock
 
 from celery.exceptions import SecurityError
 from celery.security.certificate import Certificate, CertStore, FSCertStore
+from t.unit import conftest
 
 from . import CERT1, CERT2, KEY1
 from .case import SecurityCase
@@ -84,7 +84,7 @@ class test_FSCertStore(SecurityCase):
         cert.has_expired.return_value = False
         isdir.return_value = True
         glob.return_value = ['foo.cert']
-        with mock.open():
+        with conftest.open():
             cert.get_id.return_value = 1
 
             path = os.path.join('var', 'certs')
