@@ -1,15 +1,15 @@
 import socket
 import tempfile
 from datetime import datetime, timedelta
-from unittest.mock import ANY, MagicMock, Mock, call, patch, sentinel
+from unittest.mock import ANY, MagicMock, Mock, patch, sentinel
 
 import pytest
-from case import ContextMock
 from kombu import Queue
 from kombu.exceptions import EncodeError
 
 from celery import Task, group, uuid
 from celery.app.task import _reprtask
+from celery.contrib.testing.mocks import ContextMock
 from celery.exceptions import Ignore, ImproperlyConfigured, Retry
 from celery.result import AsyncResult, EagerResult
 from celery.utils.time import parse_iso8601
@@ -572,7 +572,7 @@ class test_task_retries(TasksCase):
 
         assert task.iterations == 4
         retry_call_countdowns = [
-            call[1]['countdown'] for call in fake_retry.call_args_list
+            call_[1]['countdown'] for call_ in fake_retry.call_args_list
         ]
         assert retry_call_countdowns == [1, 2, 4, 8]
 
@@ -587,7 +587,7 @@ class test_task_retries(TasksCase):
 
         assert task.iterations == 4
         retry_call_countdowns = [
-            call[1]['countdown'] for call in fake_retry.call_args_list
+            call_[1]['countdown'] for call_ in fake_retry.call_args_list
         ]
         assert retry_call_countdowns == [0, 1, 3, 7]
 
@@ -619,7 +619,7 @@ class test_task_retries(TasksCase):
 
         assert task.iterations == 6
         retry_call_countdowns = [
-            call[1]['countdown'] for call in fake_retry.call_args_list
+            call_[1]['countdown'] for call_ in fake_retry.call_args_list
         ]
         assert retry_call_countdowns == [1, 2, 4, 8, 16, 32]
 
@@ -638,7 +638,7 @@ class test_task_retries(TasksCase):
 
         assert task.iterations == 6
         retry_call_countdowns = [
-            call[1]['countdown'] for call in fake_retry.call_args_list
+            call_[1]['countdown'] for call_ in fake_retry.call_args_list
         ]
         assert retry_call_countdowns == [1, 2, 4, 8, 16, 32]
 
@@ -650,7 +650,7 @@ class test_task_retries(TasksCase):
 
         assert task.iterations == 6
         retry_call_countdowns = [
-            call[1]['countdown'] for call in fake_retry.call_args_list
+            call_[1]['countdown'] for call_ in fake_retry.call_args_list
         ]
         assert retry_call_countdowns == [1, 2, 4, 8, 16, 16]
 
@@ -662,7 +662,7 @@ class test_task_retries(TasksCase):
 
         assert task.iterations == 6
         retry_call_countdowns = [
-            call[1]['countdown'] for call in fake_retry.call_args_list
+            call_[1]['countdown'] for call_ in fake_retry.call_args_list
         ]
         assert retry_call_countdowns == [1, 2, 4, 8, 16, 32]
 
@@ -675,7 +675,7 @@ class test_task_retries(TasksCase):
 
         assert task.iterations == 6
         retry_call_countdowns = [
-            call[1]['countdown'] for call in fake_retry.call_args_list
+            call_[1]['countdown'] for call_ in fake_retry.call_args_list
         ]
         assert retry_call_countdowns == [0, 1, 3, 7, 15, 31]
 
@@ -930,7 +930,7 @@ class test_tasks(TasksCase):
                 consumer, sresult, self.mytask.name, name='Elaine M. Benes',
             )
 
-            # With ETA.
+            # With ETA, absolute expires.
             presult2 = self.mytask.apply_async(
                 kwargs={'name': 'George Costanza'},
                 eta=self.now() + timedelta(days=1),
@@ -941,6 +941,39 @@ class test_tasks(TasksCase):
                 name='George Costanza', test_eta=True, test_expires=True,
             )
 
+            # With ETA, absolute expires without timezone.
+            presult2 = self.mytask.apply_async(
+                kwargs={'name': 'George Constanza'},
+                eta=self.now() + timedelta(days=1),
+                expires=(self.now() + timedelta(hours=2)).replace(tzinfo=None),
+            )
+            self.assert_next_task_data_equal(
+                consumer, presult2, self.mytask.name,
+                name='George Constanza', test_eta=True, test_expires=True,
+            )
+
+            # With ETA, absolute expires in the past.
+            presult2 = self.mytask.apply_async(
+                kwargs={'name': 'George Costanza'},
+                eta=self.now() + timedelta(days=1),
+                expires=self.now() - timedelta(days=2),
+            )
+            self.assert_next_task_data_equal(
+                consumer, presult2, self.mytask.name,
+                name='George Costanza', test_eta=True, test_expires=True,
+            )
+
+            # With ETA, relative expires.
+            presult2 = self.mytask.apply_async(
+                kwargs={'name': 'George Costanza'},
+                eta=self.now() + timedelta(days=1),
+                expires=2 * 24 * 60 * 60,
+            )
+            self.assert_next_task_data_equal(
+                consumer, presult2, self.mytask.name,
+                name='George Costanza', test_eta=True, test_expires=True,
+            )
+
             # With countdown.
             presult2 = self.mytask.apply_async(
                 kwargs={'name': 'George Costanza'}, countdown=10, expires=12,
@@ -998,6 +1031,11 @@ class test_tasks(TasksCase):
         with pytest.raises(Ignore):
             self.mytask.replace(sig1)
         sig1.freeze.assert_called_once_with(self.mytask.request.id)
+        sig1.set.assert_called_once_with(replaced_task_nesting=1,
+                                         chord=ANY,
+                                         group_id=ANY,
+                                         group_index=ANY,
+                                         root_id=ANY)
 
     def test_replace_with_chord(self):
         sig1 = Mock(name='sig1')
